@@ -1,17 +1,23 @@
-/* File: cmd5.c */
-
 /*
+ * File: cmd5.c
+ * Purpose: Spell and prayer casting/praying
+ *
  * Copyright (c) 1997 Ben Harrison, James E. Wilson, Robert A. Koeneke
  *
- * This software may be copied and distributed for educational, research,
- * and not for profit purposes provided that this copyright and statement
- * are included in all such copies.  Other copyrights may also apply.
+ * This work is free software; you can redistribute it and/or modify it
+ * under the terms of either:
+ *
+ * a) the GNU General Public License as published by the Free Software
+ *    Foundation, version 2, or
+ *
+ * b) the "Angband licence":
+ *    This software may be copied and distributed for educational, research,
+ *    and not for profit purposes provided that this copyright and statement
+ *    are included in all such copies.  Other copyrights may also apply.
  */
-
 #include "angband.h"
-
-#include "script.h"
-
+#include "object/tvalsval.h"
+#include "game-cmd.h"
 
 /*
  * Returns chance of failure for a spell
@@ -36,7 +42,7 @@ s16b spell_chance(int spell)
 	chance -= 3 * (p_ptr->lev - s_ptr->slevel);
 
 	/* Reduce failure rate by INT/WIS adjustment */
-	chance -= adj_mag_stat[p_ptr->stat_ind[cp_ptr->spell_stat]];
+	chance -= adj_mag_stat[p_ptr->state.stat_ind[cp_ptr->spell_stat]];
 
 	/* Not enough mana to cast */
 	if (s_ptr->smana > p_ptr->csp)
@@ -45,7 +51,7 @@ s16b spell_chance(int spell)
 	}
 
 	/* Extract the minimum failure rate */
-	minfail = adj_mag_fail[p_ptr->stat_ind[cp_ptr->spell_stat]];
+	minfail = adj_mag_fail[p_ptr->state.stat_ind[cp_ptr->spell_stat]];
 
 	/* Non mage/priest characters never get better than 5 percent */
 	if (!(cp_ptr->flags & CF_ZERO_FAIL))
@@ -54,20 +60,26 @@ s16b spell_chance(int spell)
 	}
 
 	/* Priest prayer penalty for "edged" weapons (before minfail) */
-	if (p_ptr->icky_wield)
+	if (p_ptr->state.icky_wield)
 	{
 		chance += 25;
 	}
 
-	/* Minimum failure rate */
+	/* Fear makes spells harder (before minfail) */
+	/* Note that spells that remove fear have a much lower fail rate than
+	 * surrounding spells, to make sure this doesn't cause mega fail */
+	if (p_ptr->state.afraid) chance += 20;
+
+	/* Minimal and maximal failure rate */
 	if (chance < minfail) chance = minfail;
+	if (chance > 50) chance = 50;
 
 	/* Stunning makes spells harder (after minfail) */
 	if (p_ptr->timed[TMD_STUN] > 50) chance += 25;
 	else if (p_ptr->timed[TMD_STUN]) chance += 15;
 
-	/* Amnesia makes spells fail half the time */
-	if (p_ptr->timed[TMD_AMNESIA]) chance *= 2;
+	/* Amnesia doubles failure change */
+	if (p_ptr->timed[TMD_AMNESIA]) chance = 50 + chance / 2;
 
 	/* Always a 5 percent chance of working */
 	if (chance > 95) chance = 95;
@@ -119,7 +131,7 @@ bool spell_okay(int spell, bool known, bool browse)
 /*
  * Print a list of spells (for browsing or casting or viewing).
  */
-void print_spells(const byte *spells, int num, int y, int x)
+static void print_spells(const byte *spells, int num, int y, int x)
 {
 	int i, spell;
 
@@ -199,78 +211,6 @@ void print_spells(const byte *spells, int num, int y, int x)
 
 
 
-/*
- * Hack -- display an object kind in the current window
- *
- * Include list of usable spells for readible books
- */
-void display_koff(int k_idx)
-{
-	int y;
-
-	object_type *i_ptr;
-	object_type object_type_body;
-
-	char o_name[80];
-
-
-	/* Erase the window */
-	for (y = 0; y < Term->hgt; y++)
-	{
-		/* Erase the line */
-		Term_erase(0, y, 255);
-	}
-
-	/* No info */
-	if (!k_idx) return;
-
-
-	/* Get local object */
-	i_ptr = &object_type_body;
-
-	/* Prepare the object */
-	object_wipe(i_ptr);
-
-	/* Prepare the object */
-	object_prep(i_ptr, k_idx);
-
-
-	/* Describe */
-	object_desc_spoil(o_name, sizeof(o_name), i_ptr, FALSE, 0);
-
-	/* Mention the object name */
-	Term_putstr(0, 0, -1, TERM_WHITE, o_name);
-
-
-	/* Warriors are illiterate */
-	if (!cp_ptr->spell_book) return;
-
-	/* Display spells in readible books */
-	if (i_ptr->tval == cp_ptr->spell_book)
-	{
-		int i;
-		int spell;
-		int num = 0;
-
-		byte spells[PY_MAX_SPELLS];
-
-
-		/* Extract spells */
-		for (i = 0; i < SPELLS_PER_BOOK; i++)
-		{
-			spell = get_spell_index(i_ptr, i);
-
-			/* Collect this spell */
-			if (spell >= 0) spells[num++] = spell;
-		}
-
-		/* Print spells */
-		print_spells(spells, num, 2, 0);
-	}
-}
-
-
-
 /*
  * Allow user to choose a spell/prayer from the given book.
  *
@@ -282,7 +222,7 @@ void display_koff(int k_idx)
  * The "known" should be TRUE for cast/pray, FALSE for study
  * The "browse" should be TRUE for browse, FALSE for cast/pray/study
  */
-static int get_spell(const object_type *o_ptr, cptr prompt, bool known, bool browse)
+int get_spell(const object_type *o_ptr, cptr prompt, bool known, bool browse)
 {
 	int i;
 
@@ -302,24 +242,6 @@ static int get_spell(const object_type *o_ptr, cptr prompt, bool known, bool bro
 
 	cptr p = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
 
-	int result;
-
-	/* Get the spell, if available */
-	if (repeat_pull(&result))
-	{
-		/* Verify the spell */
-		if (spell_okay(result, known, browse))
-		{
-			/* Success */
-			return (result);
-		}
-		else
-		{
-			/* Invalid repeat - reset it */
-			repeat_clear();
-		}
-	}
-
 	/* Extract spells */
 	for (i = 0; i < SPELLS_PER_BOOK; i++)
 	{
@@ -340,7 +262,12 @@ static int get_spell(const object_type *o_ptr, cptr prompt, bool known, bool bro
 	}
 
 	/* No available spells */
-	if (!okay) return (-2);
+	if (!okay)
+	{
+		msg_format("You cannot learn any %ss in that book.", p);
+		return (-2);
+	}
+
 
 
 	/* Nothing chosen yet */
@@ -350,7 +277,7 @@ static int get_spell(const object_type *o_ptr, cptr prompt, bool known, bool bro
 	redraw = FALSE;
 
 	/* Hack -- when browsing a book, start with list shown */
-	if (browse)
+	if (browse || OPT(show_lists))
 	{
 		/* Show list */
 		redraw = TRUE;
@@ -363,14 +290,15 @@ static int get_spell(const object_type *o_ptr, cptr prompt, bool known, bool bro
 	}
 
 	/* Build a prompt (accept all spells) */
-	strnfmt(out_val, 78, "(%^ss %c-%c, *=List, ESC=exit) %^s which %s? ",
-	        p, I2A(0), I2A(num - 1), prompt, p);
+	strnfmt(out_val, sizeof(out_val), "(%^ss a-%c%s, ESC=exit) %^s which %s? ",
+	        p, I2A(num - 1), (OPT(show_lists) ? "" : ", *=List"), prompt, p);
 
 	/* Get a spell from the user */
 	while (!flag && get_com(out_val, &choice))
 	{
 		/* Request redraw */
-		if ((choice == ' ') || (choice == '*') || (choice == '?'))
+		if (!OPT(show_lists) &&
+		    ((choice == ' ') || (choice == '*') || (choice == '?')))
 		{
 			/* Hide the list */
 			if (redraw)
@@ -436,7 +364,7 @@ static int get_spell(const object_type *o_ptr, cptr prompt, bool known, bool bro
 			s_ptr = &mp_ptr->info[spell];
 
 			/* Prompt */
-			strnfmt(tmp_val, 78, "%^s %s (%d mana, %d%% fail)? ",
+			strnfmt(tmp_val, sizeof(tmp_val), "%^s %s (%d mana, %d%% fail)? ",
 			        prompt, get_spell_name(cp_ptr->spell_book, spell),
 			        s_ptr->smana, spell_chance(spell));
 
@@ -460,8 +388,6 @@ static int get_spell(const object_type *o_ptr, cptr prompt, bool known, bool bro
 	/* Abort if needed */
 	if (!flag) return (-1);
 
-	repeat_push(spell);
-
 	/* Success */
 	return (spell);
 }
@@ -543,26 +469,24 @@ static void browse_spell(int spell)
 	/* Display the spell description */
 	text_out("\n\n   ");
 
-	text_out(s_text + s_info[(cp_ptr->spell_book == TV_MAGIC_BOOK) ? spell : spell + PY_MAX_SPELLS].text);
+	text_out("%s", s_text + s_info[(cp_ptr->spell_book == TV_MAGIC_BOOK) ? spell : spell + PY_MAX_SPELLS].text);
 	text_out_c(TERM_L_BLUE, "\n\n[Press any key to continue]\n");
 
 	/* Wait for input */
-	(void)inkey();
+	(void)anykey();
 
 	/* Load screen */
 	screen_load();
 }
 
 
-void do_cmd_browse_aux(const object_type *o_ptr)
+void do_cmd_browse_aux(const object_type *o_ptr, int item)
 {
 	int spell;
 
 
 	/* Track the object kind */
-	object_kind_track(o_ptr->k_idx);
-
-	/* Hack -- Handle stuff */
+	track_object(item);
 	handle_stuff();
 
 
@@ -578,177 +502,178 @@ void do_cmd_browse_aux(const object_type *o_ptr)
 	}
 }
 
-
-/*
- * Peruse the spells/prayers in a Book
- *
- * Note that *all* spells in the book are listed
- *
- * Note that browsing is allowed while confused or blind,
- * and in the dark, primarily to allow browsing in stores.
- */
-void do_cmd_browse(void)
+/* Check if the given spell is in the given book. */
+static bool spell_in_book(int spell, int book)
 {
-	int item;
+	int i;
+	object_type *o_ptr = object_from_item_idx(book);
 
-	object_type *o_ptr;
-
-	cptr q, s;
-
-
-	/* Warriors are illiterate */
-	if (!cp_ptr->spell_book)
+	for (i = 0; i < SPELLS_PER_BOOK; i++)
 	{
-		msg_print("You cannot read books!");
+		if (spell == get_spell_index(o_ptr, i))
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
+/* Gain a specific spell, specified by spell number (for mages). */
+void do_cmd_study_spell(cmd_code code, cmd_arg args[])
+{
+	int spell = args[0].choice;
+
+	int item_list[INVEN_TOTAL + MAX_FLOOR_STACK];
+	int item_num;
+	int i;
+
+	/* Check the player can study at all atm */
+	if (!player_can_study())
 		return;
-	}
 
-	/* Restrict choices to "useful" books */
-	item_tester_tval = cp_ptr->spell_book;
+	/* Check that the player can actually learn the nominated spell. */
+	item_tester_hook = obj_can_browse;
+	item_num = scan_items(item_list, N_ELEMENTS(item_list), (USE_INVEN | USE_FLOOR));
 
-	/* Get an item */
-	q = "Browse which book? ";
-	s = "You have no books that you can read.";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
-
-	/* Get the item (in the pack) */
-	if (item >= 0)
+	/* Check through all available books */
+	for (i = 0; i < item_num; i++)
 	{
-		o_ptr = &inventory[item];
-	}
+		if (spell_in_book(spell, item_list[i]))
+		{
+			if (spell_okay(spell, FALSE, FALSE))
+			{
+				/* Spell is in an available book, and player is capable. */
+				spell_learn(spell);
+				p_ptr->energy_use = 100;
+			}
+			else
+			{
+				/* Spell is present, but player incapable. */
+				msg_format("You cannot learn that spell.");
+			}
 
-	/* Get the item (on the floor) */
-	else
+			return;
+		}
+	}
+}
+
+/* Cast a spell from a book */
+void do_cmd_cast(cmd_code code, cmd_arg args[])
+{
+	int spell = args[0].choice;
+	int dir = args[1].direction;
+
+	int item_list[INVEN_TOTAL + MAX_FLOOR_STACK];
+	int item_num;
+	int i;
+
+	cptr verb = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "cast" : "recite");
+    cptr noun = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
+
+	/* Check the player can cast spells at all */
+	if (!player_can_cast())
+		return;
+
+	/* Check spell is in a book they can access */
+	item_tester_hook = obj_can_browse;
+	item_num = scan_items(item_list, N_ELEMENTS(item_list), (USE_INVEN | USE_FLOOR));
+
+	/* Check through all available books */
+	for (i = 0; i < item_num; i++)
 	{
-		o_ptr = &o_list[0 - item];
+		if (spell_in_book(spell, item_list[i]))
+		{
+			if (spell_okay(spell, TRUE, FALSE))
+			{
+				/* Get the spell */
+				const magic_type *s_ptr = &mp_ptr->info[spell];	
+				
+				/* Verify "dangerous" spells */
+				if (s_ptr->smana > p_ptr->csp)
+				{
+					/* Warning */
+					msg_format("You do not have enough mana to %s this %s.", verb, noun);
+					
+					/* Flush input */
+					flush();
+					
+					/* Verify */
+					if (!get_check("Attempt it anyway? ")) return;
+				}
+
+				/* Cast a spell */
+				if (spell_cast(spell, dir))
+					p_ptr->energy_use = 100;
+			}
+			else
+			{
+				/* Spell is present, but player incapable. */
+				msg_format("You cannot %s that %s.", verb, noun);
+			}
+
+			return;
+		}
 	}
 
-	/* Browse the book */
-	do_cmd_browse_aux(o_ptr);
 }
 
 
-
-
-/*
- * Study a book to gain a new spell/prayer
- */
-void do_cmd_study(void)
+/* Gain a random spell from the given book (for priests) */
+void do_cmd_study_book(cmd_code code, cmd_arg args[])
 {
-	int i, item;
+	int book = args[0].item;
+	object_type *o_ptr = object_from_item_idx(book);
 
-	int spell;
+	int spell = -1;
+	int i, k = 0;
 
 	cptr p = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
 
-	cptr q, s;
+	/* Check the player can study at all atm */
+	if (!player_can_study())
+		return;
 
-	object_type *o_ptr;
-
-
-	if (!cp_ptr->spell_book)
+	/* Check that the player has access to the nominated spell book. */
+	if (!item_is_available(book, obj_can_browse, (USE_INVEN | USE_FLOOR)))
 	{
-		msg_print("You cannot read books!");
+		msg_format("That item is not within your reach.");
 		return;
 	}
 
-	if (p_ptr->timed[TMD_BLIND] || no_lite())
+	/* Extract spells */
+	for (i = 0; i < SPELLS_PER_BOOK; i++)
 	{
-		msg_print("You cannot see!");
-		return;
+		int s = get_spell_index(o_ptr, i);
+		
+		/* Skip non-OK spells */
+		if (s == -1) continue;
+		if (!spell_okay(s, FALSE, FALSE)) continue;
+		
+		/* Apply the randomizer */
+		if ((++k > 1) && (randint0(k) != 0)) continue;
+		
+		/* Track it */
+		spell = s;
 	}
 
-	if (p_ptr->timed[TMD_CONFUSED])
-	{
-		msg_print("You are too confused!");
-		return;
-	}
-
-	if (!(p_ptr->new_spells))
-	{
-		msg_format("You cannot learn any new %ss!", p);
-		return;
-	}
-
-
-	/* Restrict choices to "useful" books */
-	item_tester_tval = cp_ptr->spell_book;
-
-	/* Get an item */
-	q = "Study which book? ";
-	s = "You have no books that you can read.";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
-
-	/* Get the item (in the pack) */
-	if (item >= 0)
-	{
-		o_ptr = &inventory[item];
-	}
-
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
-
-
-	/* Track the object kind */
-	object_kind_track(o_ptr->k_idx);
-
-	/* Hack -- Handle stuff */
-	handle_stuff();
-
-
-	/* Mage -- Learn a selected spell */
-	if (cp_ptr->flags & CF_CHOOSE_SPELLS)
-	{
-		/* Ask for a spell */
-		spell = get_spell(o_ptr, "study", FALSE, FALSE);
-
-		/* Allow cancel */
-		if (spell == -1) return;
-	}
-	else
-	{
-		int k = 0;
-
-		int gift = -1;
-
-		/* Extract spells */
-		for (i = 0; i < SPELLS_PER_BOOK; i++)
-		{
-			spell = get_spell_index(o_ptr, i);
-
-			/* Skip empty spells */
-			if (spell == -1) continue;
-
-			/* Skip non "okay" prayers */
-			if (!spell_okay(spell, FALSE, FALSE)) continue;
-
-			/* Apply the randomizer */
-			if ((++k > 1) && (rand_int(k) != 0)) continue;
-
-			/* Track it */
-			gift = spell;
-		}
-
-		/* Accept gift */
-		spell = gift;
-	}
-
-	/* Nothing to study */
 	if (spell < 0)
 	{
-		/* Message */
 		msg_format("You cannot learn any %ss in that book.", p);
-
-		/* Abort */
-		return;
 	}
+	else
+	{
+		spell_learn(spell);
+		p_ptr->energy_use = 100;	
+	}
+}
 
 
-	/* Take a turn */
-	p_ptr->energy_use = 100;
+/*
+ * Learn the specified spell.
+ */
+void spell_learn(int spell)
+{
+	int i;
+	cptr p = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
 
 	/* Learn the spell */
 	p_ptr->spell_flags[spell] |= PY_SPELL_LEARNED;
@@ -775,130 +700,41 @@ void do_cmd_study(void)
 	{
 		/* Message */
 		msg_format("You can learn %d more %s%s.",
-		           p_ptr->new_spells, p,
-		           (p_ptr->new_spells != 1) ? "s" : "");
+		           p_ptr->new_spells, p, PLURAL(p_ptr->new_spells));
 	}
 
 	/* Redraw Study Status */
-	p_ptr->redraw |= (PR_STUDY);
-
-	/* Redraw object recall */
-	p_ptr->window |= (PW_OBJECT);
+	p_ptr->redraw |= (PR_STUDY | PR_OBJECT);
 }
 
 
-
-/*
- * Cast a spell
- */
-void do_cmd_cast(void)
+/* Cas the specified spell */
+bool spell_cast(int spell, int dir)
 {
-	int item, spell;
 	int chance;
 
-	object_type *o_ptr;
-
-	const magic_type *s_ptr;
-
-	cptr q, s;
-
-
-	/* Require spell ability */
-	if (cp_ptr->spell_book != TV_MAGIC_BOOK)
-	{
-		msg_print("You cannot cast spells!");
-		return;
-	}
-
-	/* Require lite */
-	if (p_ptr->timed[TMD_BLIND] || no_lite())
-	{
-		msg_print("You cannot see!");
-		return;
-	}
-
-	/* Not when confused */
-	if (p_ptr->timed[TMD_CONFUSED])
-	{
-		msg_print("You are too confused!");
-		return;
-	}
-
-
-	/* Restrict choices to spell books */
-	item_tester_tval = cp_ptr->spell_book;
-
-	/* Get an item */
-	q = "Use which book? ";
-	s = "You have no spell books!";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
-
-	/* Get the item (in the pack) */
-	if (item >= 0)
-	{
-		o_ptr = &inventory[item];
-	}
-
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
-
-
-	/* Track the object kind */
-	object_kind_track(o_ptr->k_idx);
-
-	/* Hack -- Handle stuff */
-	handle_stuff();
-
-
-	/* Ask for a spell */
-	spell = get_spell(o_ptr, "cast", TRUE, FALSE);
-
-	if (spell < 0)
-	{
-		if (spell == -2) msg_print("You don't know any spells in that book.");
-		return;
-	}
-
-
 	/* Get the spell */
-	s_ptr = &mp_ptr->info[spell];
-
-
-	/* Verify "dangerous" spells */
-	if (s_ptr->smana > p_ptr->csp)
-	{
-		/* Warning */
-		msg_print("You do not have enough mana to cast this spell.");
-
-		/* Flush input */
-		flush();
-
-		/* Verify */
-		if (!get_check("Attempt it anyway? ")) return;
-	}
-
+	const magic_type *s_ptr = &mp_ptr->info[spell];	
 
 	/* Spell failure chance */
 	chance = spell_chance(spell);
 
 	/* Failed spell */
-	if (rand_int(100) < chance)
+	if (randint0(100) < chance)
 	{
-		if (flush_failure) flush();
-		msg_print("You failed to get the spell off!");
+		if (OPT(flush_failure)) flush();
+		msg_print("You failed to concentrate hard enough!");
 	}
 
 	/* Process spell */
 	else
 	{
 		/* Cast the spell */
-		if (!cast_spell(cp_ptr->spell_book, spell)) return;
+		if (!cast_spell(cp_ptr->spell_book, spell, dir)) return FALSE;
 
 		/* A spell was cast */
 		sound(MSG_SPELL);
+
 		if (!(p_ptr->spell_flags[spell] & PY_SPELL_WORKED))
 		{
 			int e = s_ptr->sexp;
@@ -910,13 +746,10 @@ void do_cmd_cast(void)
 			gain_exp(e * s_ptr->slevel);
 
 			/* Redraw object recall */
-			p_ptr->window |= (PW_OBJECT);
+			p_ptr->redraw |= (PR_OBJECT);
 		}
 	}
 
-	/* Take a turn */
-	p_ptr->energy_use = 100;
-
 	/* Sufficient mana */
 	if (s_ptr->smana <= p_ptr->csp)
 	{
@@ -937,194 +770,23 @@ void do_cmd_cast(void)
 		msg_print("You faint from the effort!");
 
 		/* Hack -- Bypass free action */
-		(void)inc_timed(TMD_PARALYZED, randint(5 * oops + 1));
+		(void)inc_timed(TMD_PARALYZED, randint1(5 * oops + 1), TRUE);
 
 		/* Damage CON (possibly permanently) */
-		if (rand_int(100) < 50)
+		if (randint0(100) < 50)
 		{
-			bool perm = (rand_int(100) < 25);
+			bool perm = (randint0(100) < 25);
 
 			/* Message */
 			msg_print("You have damaged your health!");
 
 			/* Reduce constitution */
-			(void)dec_stat(A_CON, 15 + randint(10), perm);
+			(void)dec_stat(A_CON, perm);
 		}
 	}
 
 	/* Redraw mana */
 	p_ptr->redraw |= (PR_MANA);
 
-	/* Window stuff */
-	p_ptr->window |= (PW_PLAYER_0 | PW_PLAYER_1);
-}
-
-
-/*
- * Pray a prayer
- */
-void do_cmd_pray(void)
-{
-	int item, spell, chance;
-
-	object_type *o_ptr;
-
-	const magic_type *s_ptr;
-
-	cptr q, s;
-
-
-	/* Must use prayer books */
-	if (cp_ptr->spell_book != TV_PRAYER_BOOK)
-	{
-		msg_print("Pray hard enough and your prayers may be answered.");
-		return;
-	}
-
-	/* Must have lite */
-	if (p_ptr->timed[TMD_BLIND] || no_lite())
-	{
-		msg_print("You cannot see!");
-		return;
-	}
-
-	/* Must not be confused */
-	if (p_ptr->timed[TMD_CONFUSED])
-	{
-		msg_print("You are too confused!");
-		return;
-	}
-
-
-	/* Restrict choices */
-	item_tester_tval = cp_ptr->spell_book;
-
-	/* Get an item */
-	q = "Use which book? ";
-	s = "You have no prayer books!";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
-
-	/* Get the item (in the pack) */
-	if (item >= 0)
-	{
-		o_ptr = &inventory[item];
-	}
-
-	/* Get the item (on the floor) */
-	else
-	{
-		o_ptr = &o_list[0 - item];
-	}
-
-	/* Track the object kind */
-	object_kind_track(o_ptr->k_idx);
-
-	/* Hack -- Handle stuff */
-	handle_stuff();
-
-
-	/* Choose a spell */
-	spell = get_spell(o_ptr, "recite", TRUE, FALSE);
-
-	if (spell < 0)
-	{
-		if (spell == -2) msg_print("You don't know any prayers in that book.");
-		return;
-	}
-
-
-	/* Get the spell */
-	s_ptr = &mp_ptr->info[spell];
-
-
-	/* Verify "dangerous" prayers */
-	if (s_ptr->smana > p_ptr->csp)
-	{
-		/* Warning */
-		msg_print("You do not have enough mana to recite this prayer.");
-
-		/* Flush input */
-		flush();
-
-		/* Verify */
-		if (!get_check("Attempt it anyway? ")) return;
-	}
-
-
-	/* Spell failure chance */
-	chance = spell_chance(spell);
-
-	/* Check for failure */
-	if (rand_int(100) < chance)
-	{
-		if (flush_failure) flush();
-		msg_print("You failed to concentrate hard enough!");
-	}
-
-	/* Success */
-	else
-	{
-		/* Cast the spell */
-		if (!cast_spell(cp_ptr->spell_book, spell)) return;
-
-		/* A prayer was prayed */
-		sound(MSG_PRAYER);
-		if (!(p_ptr->spell_flags[spell] & PY_SPELL_WORKED))
-		{
-			int e = s_ptr->sexp;
-
-			/* The spell worked */
-			p_ptr->spell_flags[spell] |= PY_SPELL_WORKED;
-
-			/* Gain experience */
-			gain_exp(e * s_ptr->slevel);
-
-			/* Redraw object recall */
-			p_ptr->window |= (PW_OBJECT);
-		}
-	}
-
-	/* Take a turn */
-	p_ptr->energy_use = 100;
-
-	/* Sufficient mana */
-	if (s_ptr->smana <= p_ptr->csp)
-	{
-		/* Use some mana */
-		p_ptr->csp -= s_ptr->smana;
-	}
-
-	/* Over-exert the player */
-	else
-	{
-		int oops = s_ptr->smana - p_ptr->csp;
-
-		/* No mana left */
-		p_ptr->csp = 0;
-		p_ptr->csp_frac = 0;
-
-		/* Message */
-		msg_print("You faint from the effort!");
-
-		/* Hack -- Bypass free action */
-		(void)inc_timed(TMD_PARALYZED, randint(5 * oops + 1));
-
-		/* Damage CON (possibly permanently) */
-		if (rand_int(100) < 50)
-		{
-			bool perm = (rand_int(100) < 25);
-
-			/* Message */
-			msg_print("You have damaged your health!");
-
-			/* Reduce constitution */
-			(void)dec_stat(A_CON, 15 + randint(10), perm);
-		}
-	}
-
-	/* Redraw mana */
-	p_ptr->redraw |= (PR_MANA);
-
-	/* Window stuff */
-	p_ptr->window |= (PW_PLAYER_0 | PW_PLAYER_1);
+	return TRUE;
 }
