@@ -1,16 +1,23 @@
-/* File: x-spell.c */
-
 /*
+ * File: x-spell.c
+ * Purpose: Spell effect definitions and information about them
+ *
  * Copyright (c) 1997 Ben Harrison, James E. Wilson, Robert A. Koeneke
  *
- * This software may be copied and distributed for educational, research,
- * and not for profit purposes provided that this copyright and statement
- * are included in all such copies.  Other copyrights may also apply.
+ * This work is free software; you can redistribute it and/or modify it
+ * under the terms of either:
+ *
+ * a) the GNU General Public License as published by the Free Software
+ *    Foundation, version 2, or
+ *
+ * b) the "Angband licence":
+ *    This software may be copied and distributed for educational, research,
+ *    and not for profit purposes provided that this copyright and statement
+ *    are included in all such copies.  Other copyrights may also apply.
  */
-
-
 #include "angband.h"
-#include "script.h"
+#include "effects.h"
+#include "object/tvalsval.h"
 
 
 /*
@@ -26,7 +33,7 @@
 #define SPELL_FIND_TRAPS_DOORS          4
 #define SPELL_CURE_LIGHT_WOUNDS         5
 #define SPELL_TREASURE_DETECTION        6
-#define SPELL_OBJECT_DETECTION          7
+/* #define SPELL_OBJECT_DETECTION          7 */
 #define SPELL_IDENTIFY                  8
 #define SPELL_DETECT_INVISIBLE          9
 #define SPELL_DETECT_ENCHANTMENT        10
@@ -217,7 +224,7 @@ void get_spell_info(int tval, int spell, char *p, size_t len)
 			strnfmt(p, len, " dam 2d%d", (plev / 2));
 			break; 
 		case SPELL_CURE_LIGHT_WOUNDS:
-			strnfmt(p, len, " heal 2d8");
+			strnfmt(p, len, " heal 15%%");
 			break;
 		case SPELL_STINKING_CLOUD:
 			strnfmt(p, len, " dam %d", 10 + (plev / 2));
@@ -312,7 +319,7 @@ void get_spell_info(int tval, int spell, char *p, size_t len)
 		switch (spell)
 		{
 			case PRAYER_CURE_LIGHT_WOUNDS:
-				my_strcpy(p, " heal 2d10", len);
+				my_strcpy(p, " heal 15%", len);
 				break;
 			case PRAYER_BLESS:
 				my_strcpy(p, " dur 12+d12", len);
@@ -324,7 +331,7 @@ void get_spell_info(int tval, int spell, char *p, size_t len)
 				strnfmt(p, len, " range %d", 3 * plev);
 				break;
 			case PRAYER_CURE_SERIOUS_WOUNDS:
-				my_strcpy(p, " heal 4d10", len);
+				my_strcpy(p, " heal 20%", len);
 				break;
 			case PRAYER_CHANT:
 				my_strcpy(p, " dur 24+d24", len);
@@ -337,7 +344,7 @@ void get_spell_info(int tval, int spell, char *p, size_t len)
 				        (plev / ((cp_ptr->flags & CF_BLESS_WEAPON) ? 2 : 4)));
 				break;
 			case PRAYER_CURE_CRITICAL_WOUNDS:
-				my_strcpy(p, " heal 6d10", len);
+				my_strcpy(p, " heal 25%", len);
 				break;
 			case PRAYER_SENSE_INVISIBLE:
 				my_strcpy(p, " dur 24+d24", len);
@@ -346,7 +353,7 @@ void get_spell_info(int tval, int spell, char *p, size_t len)
 				strnfmt(p, len, " dur %d+d25", 3 * plev);
 				break;
 			case PRAYER_CURE_MORTAL_WOUNDS:
-				my_strcpy(p, " heal 8d10", len);
+				my_strcpy(p, " heal 30%", len);
 				break;
 			case PRAYER_PRAYER:
 				my_strcpy(p, " dur 48+d48", len);
@@ -355,7 +362,7 @@ void get_spell_info(int tval, int spell, char *p, size_t len)
 				strnfmt(p, len, " dam d%d", 3 * plev);
 				break;
 			case PRAYER_HEAL:
-				my_strcpy(p, " heal 300", len);
+				my_strcpy(p, " heal 35%", len);
 				break;
 			case PRAYER_DISPEL_EVIL:
 				strnfmt(p, len, " dam d%d", 3 * plev);
@@ -364,10 +371,10 @@ void get_spell_info(int tval, int spell, char *p, size_t len)
 				my_strcpy(p, " heal 1000", len);
 				break;
 			case PRAYER_CURE_SERIOUS_WOUNDS2:
-				my_strcpy(p, " heal 4d10", len);
+				my_strcpy(p, " heal 20%", len);
 				break;
 			case PRAYER_CURE_MORTAL_WOUNDS2:
-				my_strcpy(p, " heal 8d10", len);
+				my_strcpy(p, " heal 30%", len);
 				break;
 			case PRAYER_HEALING:
 				my_strcpy(p, " heal 2000", len);
@@ -409,64 +416,71 @@ static void spell_wonder(int dir)
    This eliminates the worst effects later on, while
    keeping the results quite random.  It also allows
    some potent effects only at high level. */
+	effect_wonder(dir, randint1(100) + p_ptr->lev / 5, beam_chance());
+}
 
-	int py = p_ptr->py;
-	int px = p_ptr->px;
-	int plev = p_ptr->lev;
-	int die = randint(100) + plev / 5;
-	int beam = beam_chance();
 
-	if (die > 100)
-		msg_print("You feel a surge of power!");
-	if (die < 8) clone_monster(dir);
-	else if (die < 14) speed_monster(dir);
-	else if (die < 26) heal_monster(dir);
-	else if (die < 31) poly_monster(dir);
-	else if (die < 36)
-		fire_bolt_or_beam(beam - 10, GF_MISSILE, dir,
-		                  damroll(3 + ((plev - 1) / 5), 4));
-	else if (die < 41) confuse_monster(dir, plev);
-	else if (die < 46) fire_ball(GF_POIS, dir, 20 + (plev / 2), 3);
-	else if (die < 51) lite_line(dir);
-	else if (die < 56)
-		fire_beam(GF_ELEC, dir, damroll(3+((plev-5)/6), 6));
-	else if (die < 61)
-		fire_bolt_or_beam(beam-10, GF_COLD, dir,
-		                  damroll(5+((plev-5)/4), 8));
-	else if (die < 66)
-		fire_bolt_or_beam(beam, GF_ACID, dir,
-		                  damroll(6+((plev-5)/4), 8));
-	else if (die < 71)
-		fire_bolt_or_beam(beam, GF_FIRE, dir,
-		                  damroll(8+((plev-5)/4), 8));
-	else if (die < 76) drain_life(dir, 75);
-	else if (die < 81) fire_ball(GF_ELEC, dir, 30 + plev / 2, 2);
-	else if (die < 86) fire_ball(GF_ACID, dir, 40 + plev, 2);
-	else if (die < 91) fire_ball(GF_ICE, dir, 70 + plev, 3);
-	else if (die < 96) fire_ball(GF_FIRE, dir, 80 + plev, 3);
-	else if (die < 101) drain_life(dir, 100 + plev);
-	else if (die < 104) earthquake(py, px, 12);
-	else if (die < 106) destroy_area(py, px, 15, TRUE);
-	else if (die < 108) banishment();
-	else if (die < 110) dispel_monsters(120);
-	else /* RARE */
+bool spell_needs_aim(int tval, int spell)
+{
+	if (tval == TV_MAGIC_BOOK)
 	{
-		dispel_monsters(150);
-		slow_monsters();
-		sleep_monsters();
-		hp_player(300);
+		switch (spell)
+		{
+			case SPELL_MAGIC_MISSILE:
+			case SPELL_STINKING_CLOUD:
+			case SPELL_CONFUSE_MONSTER:
+			case SPELL_LIGHTNING_BOLT:
+			case SPELL_SLEEP_MONSTER:
+			case SPELL_SPEAR_OF_LIGHT:
+			case SPELL_FROST_BOLT:
+			case SPELL_TURN_STONE_TO_MUD:
+			case SPELL_WONDER:
+			case SPELL_POLYMORPH_OTHER:
+			case SPELL_FIRE_BOLT:
+			case SPELL_SLOW_MONSTER:
+			case SPELL_FROST_BALL:
+			case SPELL_TELEPORT_OTHER:
+			case SPELL_BEDLAM:
+			case SPELL_FIRE_BALL:
+			case SPELL_ACID_BOLT:
+			case SPELL_CLOUD_KILL:
+			case SPELL_ACID_BALL:
+			case SPELL_ICE_STORM:
+			case SPELL_METEOR_SWARM:
+			case SPELL_MANA_STORM:
+			case SPELL_SHOCK_WAVE:
+			case SPELL_EXPLOSION:
+			case SPELL_RIFT:
+			case SPELL_REND_SOUL: 
+			case SPELL_CHAOS_STRIKE: 
+				return TRUE;
+				
+			default:
+				return FALSE;
+		}
+	}
+	else
+	{
+		switch (spell)
+		{
+			case PRAYER_SCARE_MONSTER:
+			case PRAYER_ORB_OF_DRAINING:
+			case PRAYER_ANNIHILATION:
+			case PRAYER_TELEPORT_OTHER:
+				return TRUE;
+
+			default:
+				return FALSE;
+		}
 	}
 }
 
 
-
-static bool cast_mage_spell(int spell)
+static bool cast_mage_spell(int spell, int dir)
 {
 	int py = p_ptr->py;
 	int px = p_ptr->px;
 
-	int dir;
-
 	int plev = p_ptr->lev;
 
 	/* Hack -- chance of "beam" instead of "bolt" */
@@ -477,7 +491,6 @@ static bool cast_mage_spell(int spell)
 	{
 		case SPELL_MAGIC_MISSILE:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_bolt_or_beam(beam-10, GF_MISSILE, dir,
 			                  damroll(3 + ((plev - 1) / 5), 4));
 			break;
@@ -485,7 +498,7 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_DETECT_MONSTERS:
 		{
-			(void)detect_monsters_normal();
+			(void)detect_monsters_normal(TRUE);
 			break;
 		}
 
@@ -503,52 +516,42 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_TREASURE_DETECTION:
 		{
-			(void)detect_treasure();
-			(void)detect_objects_gold();
+			(void)detect_treasure(TRUE);
 			break;
 		}
 
 		case SPELL_CURE_LIGHT_WOUNDS:
 		{
 
-			(void)hp_player(damroll(2, 8));
-			(void)dec_timed(TMD_CUT, 15);
-			break;
-		}
-
-		case SPELL_OBJECT_DETECTION:
-		{
-			(void)detect_objects_normal();
+			(void)heal_player(15, 15);
+			(void)dec_timed(TMD_CUT, 20, TRUE);
+			(void)dec_timed(TMD_CONFUSED, 20, TRUE);
+			(void)clear_timed(TMD_BLIND, TRUE);
 			break;
 		}
 
 		case SPELL_FIND_TRAPS_DOORS:
 		{
-			(void)detect_traps();
-			(void)detect_doors();
-			(void)detect_stairs();
+			(void)detect_traps(TRUE);
+			(void)detect_doorstairs(TRUE);
 			break;
 		}
 
 		case SPELL_STINKING_CLOUD:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_ball(GF_POIS, dir, 10 + (plev / 2), 2);
 			break;
 		}
 
 		case SPELL_CONFUSE_MONSTER:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			(void)confuse_monster(dir, plev);
 			break;
 		}
 
 		case SPELL_LIGHTNING_BOLT:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
-			fire_beam(GF_ELEC, dir,
-			          damroll(3+((plev-5)/6), 6));
+			fire_beam(GF_ELEC, dir, damroll(3+((plev-5)/6), 6));
 			break;
 		}
 
@@ -560,14 +563,13 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_SLEEP_MONSTER:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			(void)sleep_monster(dir);
 			break;
 		}
 
 		case SPELL_CURE_POISON:
 		{
-			(void)clear_timed(TMD_POISONED);
+			(void)clear_timed(TMD_POISONED, TRUE);
 			break;
 		}
 
@@ -577,9 +579,8 @@ static bool cast_mage_spell(int spell)
 			break;
 		}
 
-		case SPELL_SPEAR_OF_LIGHT: /* spear of light */
+		case SPELL_SPEAR_OF_LIGHT:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			msg_print("A line of blue shimmering light appears.");
 			lite_line(dir);
 			break;
@@ -587,7 +588,6 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_FROST_BOLT:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_bolt_or_beam(beam-10, GF_COLD, dir,
 			                  damroll(5+((plev-5)/4), 8));
 			break;
@@ -595,7 +595,6 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_TURN_STONE_TO_MUD:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			(void)wall_to_mud(dir);
 			break;
 		}
@@ -611,16 +610,14 @@ static bool cast_mage_spell(int spell)
 			return recharge(2 + plev / 5);
 		}
 
-		case SPELL_WONDER: /* wonder */
+		case SPELL_WONDER:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			(void)spell_wonder(dir);
 			break;
 		}
 
 		case SPELL_POLYMORPH_OTHER:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			(void)poly_monster(dir);
 			break;
 		}
@@ -638,7 +635,6 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_FIRE_BOLT:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_bolt_or_beam(beam, GF_FIRE, dir,
 			                  damroll(6+((plev-5)/4), 8));
 			break;
@@ -646,14 +642,12 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_SLOW_MONSTER:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			(void)slow_monster(dir);
 			break;
 		}
 
 		case SPELL_FROST_BALL:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_ball(GF_COLD, dir, 30 + (plev), 2);
 			break;
 		}
@@ -665,21 +659,18 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_TELEPORT_OTHER:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			(void)teleport_monster(dir);
 			break;
 		}
 
 		case SPELL_BEDLAM:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_ball(GF_OLD_CONF, dir, plev, 4);
 			break;
 		}
 
 		case SPELL_FIRE_BALL:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_ball(GF_FIRE, dir, 55 + (plev), 2);
 			break;
 		}
@@ -728,67 +719,59 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_ACID_BOLT:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_bolt_or_beam(beam, GF_ACID, dir, damroll(8+((plev-5)/4), 8));
 			break;
 		}
 
 		case SPELL_CLOUD_KILL:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_ball(GF_POIS, dir, 40 + (plev / 2), 3);
 			break;
 		}
 
 		case SPELL_ACID_BALL:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_ball(GF_ACID, dir, 40 + (plev), 2);
 			break;
 		}
 
 		case SPELL_ICE_STORM:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_ball(GF_ICE, dir, 50 + (plev * 2), 3);
 			break;
 		}
 
 		case SPELL_METEOR_SWARM:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_swarm(2 + plev / 20, GF_METEOR, dir, 30 + plev / 2, 1);
 			break;
 		}
 
 		case SPELL_MANA_STORM:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_ball(GF_MANA, dir, 300 + (plev * 2), 3);
 			break;
 		}
 		case SPELL_DETECT_INVISIBLE:
 		{
-			(void)detect_monsters_invis();
+			(void)detect_monsters_invis(TRUE);
 			break;
 		}
 
 		case SPELL_DETECT_ENCHANTMENT:
 		{
-			(void)detect_objects_magic();
+			(void)detect_objects_magic(TRUE);
 			break;
 		}
 
 		case SPELL_SHOCK_WAVE:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_ball(GF_SOUND, dir, 10 + plev, 2);
 			break;
 		}
 
 		case SPELL_EXPLOSION:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_ball(GF_SHARD, dir, 20 + (plev * 2), 2);
 			break;
 		}
@@ -801,13 +784,13 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_RESIST_FIRE:
 		{
-			(void)inc_timed(TMD_OPP_FIRE, randint(20) + 20);
+			(void)inc_timed(TMD_OPP_FIRE, randint1(20) + 20, TRUE);
 			break;
 		}
 
 		case SPELL_RESIST_COLD:
 		{
-			(void)inc_timed(TMD_OPP_COLD, randint(20) + 20);
+			(void)inc_timed(TMD_OPP_COLD, randint1(20) + 20, TRUE);
 			break;
 		}
 
@@ -818,40 +801,40 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_RESIST_POISON:
 		{
-			(void)inc_timed(TMD_OPP_POIS, randint(20) + 20);
+			(void)inc_timed(TMD_OPP_POIS, randint1(20) + 20, TRUE);
 			break;
 		}
 
 		case SPELL_RESISTANCE:
 		{
-			int time = randint(20) + 20;
-			(void)inc_timed(TMD_OPP_ACID, time);
-			(void)inc_timed(TMD_OPP_ELEC, time);
-			(void)inc_timed(TMD_OPP_FIRE, time);
-			(void)inc_timed(TMD_OPP_COLD, time);
-			(void)inc_timed(TMD_OPP_POIS, time);
+			int time = randint1(20) + 20;
+			(void)inc_timed(TMD_OPP_ACID, time, TRUE);
+			(void)inc_timed(TMD_OPP_ELEC, time, TRUE);
+			(void)inc_timed(TMD_OPP_FIRE, time, TRUE);
+			(void)inc_timed(TMD_OPP_COLD, time, TRUE);
+			(void)inc_timed(TMD_OPP_POIS, time, TRUE);
 			break;
 		}
 
 		case SPELL_HEROISM:
 		{
 			(void)hp_player(10);
-			(void)inc_timed(TMD_HERO, randint(25) + 25);
-			(void)clear_timed(TMD_AFRAID);
+			(void)inc_timed(TMD_HERO, randint1(25) + 25, TRUE);
+			(void)clear_timed(TMD_AFRAID, TRUE);
 			break;
 		}
 
 		case SPELL_SHIELD:
 		{
-			(void)inc_timed(TMD_SHIELD, randint(20) + 30);
+			(void)inc_timed(TMD_SHIELD, randint1(20) + 30, TRUE);
 			break;
 		}
 
 		case SPELL_BERSERKER:
 		{
 			(void)hp_player(30);
-			(void)inc_timed(TMD_SHERO, randint(25) + 25);
-			(void)clear_timed(TMD_AFRAID);
+			(void)inc_timed(TMD_SHERO, randint1(25) + 25, TRUE);
+			(void)clear_timed(TMD_AFRAID, TRUE);
 			break;
 		}
 
@@ -859,32 +842,29 @@ static bool cast_mage_spell(int spell)
 		{
 			if (!p_ptr->timed[TMD_FAST])
 			{
-				(void)set_timed(TMD_FAST, randint(20) + plev);
+				(void)set_timed(TMD_FAST, randint1(20) + plev, TRUE);
 			}
 			else
 			{
-				(void)inc_timed(TMD_FAST, randint(5));
+				(void)inc_timed(TMD_FAST, randint1(5), TRUE);
 			}
 			break;
 		}
 
 		case SPELL_RIFT:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_beam(GF_GRAVITY, dir,	40 + damroll(plev, 7));
 			break;
 		}
 
 		case SPELL_REND_SOUL: /* rend soul */
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_bolt_or_beam(beam / 4, GF_NETHER, dir, damroll(11, plev));
 			break;
 		}
 
 		case SPELL_CHAOS_STRIKE: /* chaos strike */
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_bolt_or_beam(beam, GF_CHAOS, dir, damroll(13, plev));
 			break;
 		}
@@ -897,13 +877,13 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_ENCHANT_ARMOR: /* enchant armor */
 		{
-			return enchant_spell(0, 0, rand_int(3) + plev / 20);
+			return enchant_spell(0, 0, randint0(3) + plev / 20);
 		}
 
 		case SPELL_ENCHANT_WEAPON: /* enchant weapon */
 		{
-			return enchant_spell(rand_int(4) + plev / 20,
-			                     rand_int(4) + plev / 20, 0);
+			return enchant_spell(randint0(4) + plev / 20,
+			                     randint0(4) + plev / 20, 0);
 		}
 	}
 
@@ -912,39 +892,41 @@ static bool cast_mage_spell(int spell)
 }
 
 
-static bool cast_priest_spell(int spell)
+static bool cast_priest_spell(int spell, int dir)
 {
 	int py = p_ptr->py;
 	int px = p_ptr->px;
 
-	int dir;
-
 	int plev = p_ptr->lev;
 
+	int amt;
+
 	switch (spell)
 	{
 		case PRAYER_DETECT_EVIL:
 		{
-			(void)detect_monsters_evil();
+			(void)detect_monsters_evil(TRUE);
 			break;
 		}
 
 		case PRAYER_CURE_LIGHT_WOUNDS:
 		{
-			(void)hp_player(damroll(2, 10));
-			(void)dec_timed(TMD_CUT, 10);
+			(void)heal_player(15, 15);
+			(void)dec_timed(TMD_CUT, 20, TRUE);
+			(void)dec_timed(TMD_CONFUSED, 20, TRUE);
+			(void)clear_timed(TMD_BLIND, TRUE);
 			break;
 		}
 
 		case PRAYER_BLESS:
 		{
-			(void)inc_timed(TMD_BLESSED, randint(12) + 12);
+			(void)inc_timed(TMD_BLESSED, randint1(12) + 12, TRUE);
 			break;
 		}
 
 		case PRAYER_REMOVE_FEAR:
 		{
-			(void)clear_timed(TMD_AFRAID);
+			(void)clear_timed(TMD_AFRAID, TRUE);
 			break;
 		}
 
@@ -956,26 +938,24 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_FIND_TRAPS:
 		{
-			(void)detect_traps();
+			(void)detect_traps(TRUE);
 			break;
 		}
 
 		case PRAYER_DETECT_DOORS_STAIRS:
 		{
-			(void)detect_doors();
-			(void)detect_stairs();
+			(void)detect_doorstairs(TRUE);
 			break;
 		}
 
 		case PRAYER_SLOW_POISON:
 		{
-			(void)set_timed(TMD_POISONED, p_ptr->timed[TMD_POISONED] / 2);
+			(void)set_timed(TMD_POISONED, p_ptr->timed[TMD_POISONED] / 2, TRUE);
 			break;
 		}
 
 		case PRAYER_SCARE_MONSTER:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			(void)fear_monster(dir, plev);
 			break;
 		}
@@ -988,14 +968,16 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_CURE_SERIOUS_WOUNDS:
 		{
-			(void)hp_player(damroll(4, 10));
-			(void)set_timed(TMD_CUT, (p_ptr->timed[TMD_CUT] / 2) - 20);
+			(void)heal_player(20, 25);
+			(void)clear_timed(TMD_CUT, TRUE);
+			(void)clear_timed(TMD_CONFUSED, TRUE);
+			(void)clear_timed(TMD_BLIND, TRUE);
 			break;
 		}
 
 		case PRAYER_CHANT:
 		{
-			(void)inc_timed(TMD_BLESSED, randint(24) + 24);
+			(void)inc_timed(TMD_BLESSED, randint1(24) + 24, TRUE);
 			break;
 		}
 
@@ -1019,20 +1001,19 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_RESIST_HEAT_COLD:
 		{
-			(void)inc_timed(TMD_OPP_FIRE, randint(10) + 10);
-			(void)inc_timed(TMD_OPP_COLD, randint(10) + 10);
+			(void)inc_timed(TMD_OPP_FIRE, randint1(10) + 10, TRUE);
+			(void)inc_timed(TMD_OPP_COLD, randint1(10) + 10, TRUE);
 			break;
 		}
 
 		case PRAYER_NEUTRALIZE_POISON:
 		{
-			(void)clear_timed(TMD_POISONED);
+			(void)clear_timed(TMD_POISONED, TRUE);
 			break;
 		}
 
 		case PRAYER_ORB_OF_DRAINING:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			fire_ball(GF_HOLY_ORB, dir,
 			          (damroll(3, 6) + plev +
 			           (plev / ((cp_ptr->flags & CF_BLESS_WEAPON) ? 2 : 4))),
@@ -1042,21 +1023,25 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_CURE_CRITICAL_WOUNDS:
 		{
-			(void)hp_player(damroll(6, 10));
-			(void)clear_timed(TMD_CUT);
-			(void)clear_timed(TMD_AMNESIA);
+			(void)heal_player(25, 30);
+			(void)clear_timed(TMD_CUT, TRUE);
+			(void)clear_timed(TMD_AMNESIA, TRUE);
+			(void)clear_timed(TMD_CONFUSED, TRUE);
+			(void)clear_timed(TMD_BLIND, TRUE);
+			(void)clear_timed(TMD_POISONED, TRUE);
+			(void)clear_timed(TMD_STUN, TRUE);
 			break;
 		}
 
 		case PRAYER_SENSE_INVISIBLE:
 		{
-			(void)inc_timed(TMD_SINVIS, randint(24) + 24);
+			(void)inc_timed(TMD_SINVIS, randint1(24) + 24, TRUE);
 			break;
 		}
 
 		case PRAYER_PROTECTION_FROM_EVIL:
 		{
-			(void)inc_timed(TMD_PROTEVIL, randint(25) + 3 * p_ptr->lev);
+			(void)inc_timed(TMD_PROTEVIL, randint1(25) + 3 * p_ptr->lev, TRUE);
 			break;
 		}
 
@@ -1074,9 +1059,13 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_CURE_MORTAL_WOUNDS:
 		{
-			(void)hp_player(damroll(8, 10));
-			(void)clear_timed(TMD_STUN);
-			(void)clear_timed(TMD_CUT);
+			(void)heal_player(30, 50);
+			(void)clear_timed(TMD_CUT, TRUE);
+			(void)clear_timed(TMD_AMNESIA, TRUE);
+			(void)clear_timed(TMD_CONFUSED, TRUE);
+			(void)clear_timed(TMD_BLIND, TRUE);
+			(void)clear_timed(TMD_POISONED, TRUE);
+			(void)clear_timed(TMD_STUN, TRUE);
 			break;
 		}
 
@@ -1088,27 +1077,34 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_PRAYER:
 		{
-			(void)inc_timed(TMD_BLESSED, randint(48) + 48);
+			(void)inc_timed(TMD_BLESSED, randint1(48) + 48, TRUE);
 			break;
 		}
 
 		case PRAYER_DISPEL_UNDEAD:
 		{
-			(void)dispel_undead(randint(plev * 3));
+			(void)dispel_undead(randint1(plev * 3));
 			break;
 		}
 
 		case PRAYER_HEAL:
 		{
-			(void)hp_player(300);
-			(void)clear_timed(TMD_STUN);
-			(void)clear_timed(TMD_CUT);
+			amt = (p_ptr->mhp * 35) / 100;
+                        if (amt < 300) amt = 300;
+			
+			(void)hp_player(amt);
+			(void)clear_timed(TMD_CUT, TRUE);
+			(void)clear_timed(TMD_AMNESIA, TRUE);
+			(void)clear_timed(TMD_CONFUSED, TRUE);
+			(void)clear_timed(TMD_BLIND, TRUE);
+			(void)clear_timed(TMD_POISONED, TRUE);
+			(void)clear_timed(TMD_STUN, TRUE);
 			break;
 		}
 
 		case PRAYER_DISPEL_EVIL:
 		{
-			(void)dispel_evil(randint(plev * 3));
+			(void)dispel_evil(randint1(plev * 3));
 			break;
 		}
 
@@ -1120,24 +1116,24 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_HOLY_WORD:
 		{
-			(void)dispel_evil(randint(plev * 4));
+			(void)dispel_evil(randint1(plev * 4));
 			(void)hp_player(1000);
-			(void)clear_timed(TMD_AFRAID);
-			(void)clear_timed(TMD_POISONED);
-			(void)clear_timed(TMD_STUN);
-			(void)clear_timed(TMD_CUT);
+			(void)clear_timed(TMD_AFRAID, TRUE);
+			(void)clear_timed(TMD_POISONED, TRUE);
+			(void)clear_timed(TMD_STUN, TRUE);
+			(void)clear_timed(TMD_CUT, TRUE);
 			break;
 		}
 
 		case PRAYER_DETECT_MONSTERS:
 		{
-			(void)detect_monsters_normal();
+			(void)detect_monsters_normal(TRUE);
 			break;
 		}
 
 		case PRAYER_DETECTION:
 		{
-			(void)detect_all();
+			(void)detect_all(TRUE);
 			break;
 		}
 
@@ -1160,24 +1156,30 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_CURE_SERIOUS_WOUNDS2:
 		{
-			(void)hp_player(damroll(4, 10));
-			(void)clear_timed(TMD_CUT);
+			(void)heal_player(20, 25);
+			(void)clear_timed(TMD_CUT, TRUE);
+			(void)clear_timed(TMD_CONFUSED, TRUE);
+			(void)clear_timed(TMD_BLIND, TRUE);
 			break;
 		}
 
 		case PRAYER_CURE_MORTAL_WOUNDS2:
 		{
-			(void)hp_player(damroll(8, 10));
-			(void)clear_timed(TMD_STUN);
-			(void)clear_timed(TMD_CUT);
+			(void)heal_player(30, 50);
+			(void)clear_timed(TMD_CUT, TRUE);
+			(void)clear_timed(TMD_AMNESIA, TRUE);
+			(void)clear_timed(TMD_CONFUSED, TRUE);
+			(void)clear_timed(TMD_BLIND, TRUE);
+			(void)clear_timed(TMD_POISONED, TRUE);
+			(void)clear_timed(TMD_STUN, TRUE);
 			break;
 		}
 
 		case PRAYER_HEALING:
 		{
 			(void)hp_player(2000);
-			(void)clear_timed(TMD_STUN);
-			(void)clear_timed(TMD_CUT);
+			(void)clear_timed(TMD_STUN, TRUE);
+			(void)clear_timed(TMD_CUT, TRUE);
 			break;
 		}
 
@@ -1200,13 +1202,13 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_DISPEL_UNDEAD2:
 		{
-			(void)dispel_undead(randint(plev * 4));
+			(void)dispel_undead(randint1(plev * 4));
 			break;
 		}
 
 		case PRAYER_DISPEL_EVIL2:
 		{
-			(void)dispel_evil(randint(plev * 4));
+			(void)dispel_evil(randint1(plev * 4));
 			break;
 		}
 
@@ -1227,7 +1229,6 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_ANNIHILATION:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			drain_life(dir, 200);
 			break;
 		}
@@ -1251,12 +1252,12 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_ENCHANT_WEAPON:
 		{
-			return enchant_spell(rand_int(4) + 1, rand_int(4) + 1, 0);
+			return enchant_spell(randint0(4) + 1, randint0(4) + 1, 0);
 		}
 
 		case PRAYER_ENCHANT_ARMOUR:
 		{
-			return enchant_spell(0, 0, rand_int(3) + 2);
+			return enchant_spell(0, 0, randint0(3) + 2);
 		}
 
 		case PRAYER_ELEMENTAL_BRAND:
@@ -1279,7 +1280,6 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_TELEPORT_OTHER:
 		{
-			if (!get_aim_dir(&dir)) return (FALSE);
 			(void)teleport_monster(dir);
 			break;
 		}
@@ -1312,14 +1312,14 @@ static bool cast_priest_spell(int spell)
 }
 
 
-bool cast_spell(int tval, int index)
+bool cast_spell(int tval, int index, int dir)
 {
 	if (tval == TV_MAGIC_BOOK)
 	{
-		return cast_mage_spell(index);
+		return cast_mage_spell(index, dir);
 	}
 	else
 	{
-		return cast_priest_spell(index);
+		return cast_priest_spell(index, dir);
 	}
 }
